/*******************************************************************************
 * This file is part of logisim-evolution.
 *
 *   logisim-evolution is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   logisim-evolution is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with logisim-evolution.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   Original code by Carl Burch (http://www.cburch.com), 2011.
 *   Subsequent modifications by :
 *     + Haute École Spécialisée Bernoise
 *       http://www.bfh.ch
 *     + Haute École du paysage, d'ingénierie et d'architecture de Genève
 *       http://hepia.hesge.ch/
 *     + Haute École d'Ingénierie et de Gestion du Canton de Vaud
 *       http://www.heig-vd.ch/
 *   The project is currently maintained by :
 *     + REDS Institute - HEIG-VD
 *       Yverdon-les-Bains, Switzerland
 *       http://reds.heig-vd.ch
 *******************************************************************************/
package com.cburch.logisim.std.plexers;

import java.util.ArrayList;
import java.util.SortedMap;
import java.util.TreeMap;

import com.bfh.logisim.designrulecheck.Netlist;
import com.bfh.logisim.designrulecheck.NetlistComponent;
import com.bfh.logisim.fpgagui.FPGAReport;
import com.bfh.logisim.hdlgenerator.AbstractHDLGeneratorFactory;
import com.bfh.logisim.settings.Settings;
import com.cburch.logisim.data.AttributeSet;

public class DecoderHDLGeneratorFactory extends AbstractHDLGeneratorFactory {

	@Override
	public String getComponentStringIdentifier() {
		return "DECODER";
	}

	@Override
	public SortedMap<String, Integer> GetInputList(Netlist TheNetlist,
			AttributeSet attrs) {
		SortedMap<String, Integer> Inputs = new TreeMap<String, Integer>();
		Inputs.put("Enable", 1);
		Inputs.put("Sel", attrs.getValue(Plexers.ATTR_SELECT).getWidth());
		return Inputs;
	}

	@Override
	public ArrayList<String> GetModuleFunctionality(Netlist TheNetlist,
			AttributeSet attrs, FPGAReport Reporter, String HDLType) {
		ArrayList<String> Contents = new ArrayList<String>();
		int nr_of_select_bits = attrs.getValue(Plexers.ATTR_SELECT).getWidth();
		int num_outputs = (1 << nr_of_select_bits);
		String Space = " ";
		for (int i = 0; i < num_outputs; i++) {
			String binValue = IntToBin(i, nr_of_select_bits, HDLType);
			if (i == 10)
				Space = "";
			if (HDLType.equals(Settings.VHDL)) {
				Contents.add("   DecoderOut_" + i + Space
						+ "<= '1' WHEN sel = " + binValue + " AND");
				Contents.add(Space
						+ "                             Enable = '1' ELSE '0';");
			} else {
				Contents.add("   assign DecoderOut_" + Integer.toString(i)
						+ Space + " = (Enable&(sel == " + binValue
						+ ")) ? 1'b1 : 1'b0;");
			}
		}
		return Contents;
	}

	@Override
	public SortedMap<String, Integer> GetOutputList(Netlist TheNetlist,
			AttributeSet attrs) {
		SortedMap<String, Integer> Outputs = new TreeMap<String, Integer>();
		for (int i = 0; i < (1 << attrs.getValue(Plexers.ATTR_SELECT)
				.getWidth()); i++) {
			Outputs.put("DecoderOut_" + Integer.toString(i), 1);
		}
		return Outputs;
	}

	@Override
	public SortedMap<String, String> GetPortMap(Netlist Nets,
			NetlistComponent ComponentInfo, FPGAReport Reporter, String HDLType) {
		SortedMap<String, String> PortMap = new TreeMap<String, String>();
		int nr_of_select_bits = ComponentInfo.GetComponent().getAttributeSet()
				.getValue(Plexers.ATTR_SELECT).getWidth();
		int select_input_index = (1 << nr_of_select_bits);
		// first outputs
		for (int i = 0; i < select_input_index; i++)
			PortMap.putAll(GetNetMap("DecoderOut_" + Integer.toString(i), true,
					ComponentInfo, i, Reporter, HDLType, Nets));
		// select..
		PortMap.putAll(GetNetMap("Sel", true, ComponentInfo,
				select_input_index, Reporter, HDLType, Nets));

		// now connect enable input...
		if (ComponentInfo.GetComponent().getAttributeSet()
				.getValue(Plexers.ATTR_ENABLE).booleanValue()) {
			PortMap.putAll(GetNetMap("Enable", false, ComponentInfo,
					select_input_index + 1, Reporter, HDLType, Nets));
		} else {
			String SetBit = (HDLType.equals(Settings.VHDL)) ? "'1'" : "1'b1";
			PortMap.put("Enable", SetBit);
		}
		return PortMap;
	}

	@Override
	public String GetSubDir() {
		return "plexers";
	}

	@Override
	public boolean HDLTargetSupported(String HDLType, AttributeSet attrs,
			char Vendor) {
		return true;
	}

}
